<?php /**
 * @file
 * Contains \Drupal\scheduled_maintenance\EventSubscriber\InitSubscriber.
 */

namespace Drupal\scheduled_maintenance\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class InitSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [KernelEvents::REQUEST => ['onEvent', 0]];
  }

  public function onEvent() {
    // Get variable values.
    $maintenance_mode = \Drupal::state()->get('system.maintenance_mode');
    $time = \Drupal::config('scheduled_maintenance.settings')->get('time');

    // Validate that maintenance mode is not enabled and that a maintenance time
    // is set.
    if ($maintenance_mode || !$time) {
      return;
    }

    // Validate the date and time format.
    if (!_scheduled_maintenance_validate_date($time)) {
      return;
    }

    // Convert time to timestamp.
    $timestamp = strtotime($time);

    // If maintenance time is less than or equal to the current time, enable
    // maintenance mode and delete the maintenance time variable.
    if ($timestamp <= REQUEST_TIME) {
      \Drupal::state()->set('system.maintenance_mode', 1);
      \Drupal::service('config.factory')->getEditable('scheduled_maintenance.settings')->clear('time')->save();
    }
    else {
      // Get message variable value.
      $message = \Drupal::config('scheduled_maintenance.settings')->get('message');

      // Validate that there is a message set.
      if (!$message) {
        return;
      }

      // Get message time variable values.
      $offset_value = \Drupal::config('scheduled_maintenance.settings')->get('offset.value');
      $offset_unit = \Drupal::config('scheduled_maintenance.settings')->get('offset.unit');

      // Validate message time offset unit to one of the allowed units.
      if (!$offset_unit || !array_key_exists($offset_unit, _scheduled_maintenance_get_allowed_units())) {
        return;
      }

      // Validate message time offset value to a positive integer.
      if ($offset_value === '' || (!is_numeric($offset_value) || intval($offset_value) != $offset_value || $offset_value <= 0)) {
        return;
      }

      // Convert message time offset to timestamp.
      $message_timestamp = strtotime("-$offset_value $offset_unit", $timestamp);

      // If message time is less than or equal to the current time, show the
      // message.
      if ($message_timestamp <= REQUEST_TIME) {
        drupal_set_message(t('@maintenance_warning_message', [
          '@maintenance_warning_message' => $message
          ]), 'warning', FALSE);
      }
    }
  }

}
